@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2022 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
 import re
-from gi.repository import GLib, Gtk
+from gi.repository import GLib, Gtk  # @UnresolvedImport
 
 from xpra.util import (
     CLIENT_EXIT,
@@ -13,6 +14,7 @@ from xpra.util import (
     ellipsizer, repr_ellipsized, reverse_dict, typedict,
     )
 from xpra.os_util import bytestostr, OSX, WIN32
+from xpra.common import RESOLUTION_ALIASES
 from xpra.client.gtk_base.menu_helper import (
     MenuHelper,
     BANDWIDTH_MENU_OPTIONS,
@@ -49,10 +51,15 @@ SHOW_TRANSFERS = envbool("XPRA_SHOW_TRANSFERS", True)
 SHOW_CLIPBOARD_MENU = envbool("XPRA_SHOW_CLIPBOARD_MENU", True)
 SHOW_CLOSE = envbool("XPRA_SHOW_CLOSE", True)
 SHOW_SHUTDOWN = envbool("XPRA_SHOW_SHUTDOWN", True)
+MONITORS_MENU = envbool("XPRA_SHOW_MONITORS_MENU", True)
 WINDOWS_MENU = envbool("XPRA_SHOW_WINDOWS_MENU", True)
 START_MENU = envbool("XPRA_SHOW_START_MENU", True)
 MENU_ICONS = envbool("XPRA_MENU_ICONS", True)
 
+FULL_LAYOUT_LIST = envbool("XPRA_FULL_LAYOUT_LIST", True)
+
+NEW_MONITOR_RESOLUTIONS = os.environ.get("XPRA_NEW_MONITOR_RESOLUTIONS",
+                                         "640x480,1024x768,1600x1200,FHD,4K").split(",")
 
 CLIPBOARD_LABELS = ["Clipboard", "Primary", "Secondary"]
 CLIPBOARD_LABEL_TO_NAME = {
@@ -82,12 +89,15 @@ class GTKTrayMenuBase(MenuHelper):
     def do_setup_menu(self, show_close):
         log("setup_menu(%s)", show_close)
         menu = Gtk.Menu()
+        def add(menuitem):
+            if menuitem:
+                menu.append(menuitem)
         title_item = None
         if SHOW_TITLE_ITEM:
             title_item = Gtk.MenuItem()
             title_item.set_label(self.client.session_name or "Xpra")
             set_sensitive(title_item, False)
-            menu.append(title_item)
+            add(title_item)
             def set_menu_title(*_args):
                 #set the real name when available:
                 try:
@@ -95,48 +105,51 @@ class GTKTrayMenuBase(MenuHelper):
                 except Exception:
                     title = self.client.session_name or "Xpra"
                 title_item.set_label(title)
-            self.client.after_handshake(set_menu_title)
-
-        menu.append(self.make_infomenuitem())
-        menu.append(self.make_featuresmenuitem())
+            self.after_handshake(set_menu_title)
+        add(self.make_infomenuitem())
+        add(self.make_featuresmenuitem())
         if mixin_features.windows and self.client.keyboard_helper:
-            menu.append(self.make_keyboardmenuitem())
+            add(self.make_keyboardmenuitem())
         if mixin_features.clipboard and SHOW_CLIPBOARD_MENU:
-            menu.append(self.make_clipboardmenuitem())
+            add(self.make_clipboardmenuitem())
         if mixin_features.windows:
-            menu.append(self.make_picturemenuitem())
+            add(self.make_picturemenuitem())
         if mixin_features.audio and STARTSTOP_SOUND_MENU:
-            menu.append(self.make_audiomenuitem())
+            add(self.make_audiomenuitem())
         if mixin_features.webcam and WEBCAM_MENU:
-            menu.append(self.make_webcammenuitem())
+            add(self.make_webcammenuitem())
+        if mixin_features.display and MONITORS_MENU:
+            add(self.make_monitorsmenuitem())
         if mixin_features.windows and WINDOWS_MENU:
-            menu.append(self.make_windowsmenuitem())
+            add(self.make_windowsmenuitem())
         if RUNCOMMAND_MENU or SHOW_SERVER_COMMANDS or SHOW_UPLOAD or SHOW_SHUTDOWN:
-            menu.append(self.make_servermenuitem())
+            add(self.make_servermenuitem())
         if mixin_features.windows and START_MENU:
-            menu.append(self.make_startmenuitem())
-        menu.append(self.make_disconnectmenuitem())
+            add(self.make_startmenuitem())
+        add(self.make_disconnectmenuitem())
         if show_close:
-            menu.append(self.make_closemenuitem())
+            add(self.make_closemenuitem())
         menu.connect("deactivate", self.menu_deactivated)
         menu.show_all()
         log("setup_menu(%s) done", show_close)
         return menu
 
-
     def make_infomenuitem(self):
         info_menu_item = self.menuitem("Information", "information.png")
         menu = Gtk.Menu()
         info_menu_item.set_submenu(menu)
-        menu.append(self.make_aboutmenuitem())
-        menu.append(self.make_sessioninfomenuitem())
+        def add(menuitem):
+            if menuitem:
+                menu.append(menuitem)
+        add(self.make_aboutmenuitem())
+        add(self.make_sessioninfomenuitem())
         if SHOW_QR:
-            menu.append(self.make_qrmenuitem())
+            add(self.make_qrmenuitem())
         if SHOW_VERSION_CHECK:
-            menu.append(self.make_updatecheckmenuitem())
-        menu.append(self.make_bugreportmenuitem())
-        menu.append(self.make_docsmenuitem())
-        menu.append(self.make_html5menuitem())
+            add(self.make_updatecheckmenuitem())
+        add(self.make_bugreportmenuitem())
+        add(self.make_docsmenuitem())
+        add(self.make_html5menuitem())
         info_menu_item.show_all()
         return info_menu_item
 
@@ -185,7 +198,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.sharing_menuitem.set_tooltip_text("Sharing cannot be changed on this server")
             else:
                 self.sharing_menuitem.set_tooltip_text("")
-        self.client.after_handshake(set_sharing_menuitem)
+        self.after_handshake(set_sharing_menuitem)
         self.client.on_server_setting_changed("sharing", set_sharing_menuitem)
         self.client.on_server_setting_changed("sharing-toggle", set_sharing_menuitem)
         return self.sharing_menuitem
@@ -211,7 +224,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.lock_menuitem.set_tooltip_text("Session locking cannot be toggled on this server")
             else:
                 self.lock_menuitem.set_tooltip_text("")
-        self.client.after_handshake(set_lock_menuitem)
+        self.after_handshake(set_lock_menuitem)
         self.client.on_server_setting_changed("lock", set_lock_menuitem)
         self.client.on_server_setting_changed("lock-toggle", set_lock_menuitem)
         return self.lock_menuitem
@@ -232,7 +245,7 @@ class GTKTrayMenuBase(MenuHelper):
             else:
                 self.readonly_menuitem.set_tooltip_text("Cannot disable readonly mode: "+
                                                         "the server has locked the session to read only")
-        self.client.after_handshake(set_readonly_menuitem)
+        self.after_handshake(set_readonly_menuitem)
         return self.readonly_menuitem
 
     def make_bellmenuitem(self):
@@ -258,7 +271,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.bell_menuitem.set_tooltip_text("Forward system bell")
             else:
                 self.bell_menuitem.set_tooltip_text("Cannot forward the system bell: the feature has been disabled")
-        self.client.after_handshake(set_bell_menuitem)
+        self.after_handshake(set_bell_menuitem)
         self.client.on_server_setting_changed("bell", set_bell_menuitem)
         return  self.bell_menuitem
 
@@ -284,7 +297,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.cursors_menuitem.set_tooltip_text("Forward custom mouse cursors")
             else:
                 self.cursors_menuitem.set_tooltip_text("Cannot forward mouse cursors: the feature has been disabled")
-        self.client.after_handshake(set_cursors_menuitem)
+        self.after_handshake(set_cursors_menuitem)
         self.client.on_server_setting_changed("cursors", set_cursors_menuitem)
         return  self.cursors_menuitem
 
@@ -308,7 +321,7 @@ class GTKTrayMenuBase(MenuHelper):
             else:
                 self.notifications_menuitem.set_tooltip_text("Cannot forward system notifications: "+
                                                              "the feature has been disabled")
-        self.client.after_handshake(set_notifications_menuitem)
+        self.after_handshake(set_notifications_menuitem)
         return self.notifications_menuitem
 
 
@@ -424,7 +437,7 @@ class GTKTrayMenuBase(MenuHelper):
             #connect signals:
             for direction_item in items:
                 direction_item.connect("toggled", self.clipboard_direction_changed, clipboard_submenu)
-        self.client.after_handshake(set_clipboard_menu)
+        self.after_handshake(set_clipboard_menu)
         return self.clipboard_menuitem
 
 
@@ -458,7 +471,7 @@ class GTKTrayMenuBase(MenuHelper):
                 log("set_keyboard_sync_menuitem%s enabled=%s", args, kh.keyboard_sync)
             self.keyboard_sync_menuitem.set_active(kh and bool(kh.keyboard_sync))
             set_keyboard_sync_tooltip()
-        self.client.after_handshake(set_keyboard_sync_menuitem)
+        self.after_handshake(set_keyboard_sync_menuitem)
         return self.keyboard_sync_menuitem
 
     def make_shortcutsmenuitem(self):
@@ -489,7 +502,7 @@ class GTKTrayMenuBase(MenuHelper):
                 log("opengl_toggled%s", args)
                 self.client.toggle_opengl()
             gl.connect("toggled", opengl_toggled)
-        self.client.after_handshake(gl_set)
+        self.after_handshake(gl_set)
         return gl
 
     def make_modalwindowmenuitem(self):
@@ -502,7 +515,7 @@ class GTKTrayMenuBase(MenuHelper):
             log("modal_toggled%s modal_windows=%s", args, self.client.modal_windows)
         def set_modal_menuitem(*_args):
             set_sensitive(modal, True)
-        self.client.after_handshake(set_modal_menuitem)
+        self.after_handshake(set_modal_menuitem)
         modal.connect("toggled", modal_toggled)
         return modal
 
@@ -566,7 +579,7 @@ class GTKTrayMenuBase(MenuHelper):
                     set_sensitive(c, below_server_limit)
                     if not below_server_limit:
                         c.set_tooltip_text("server set the limit to %sbps" % std_unit_dec(sbl))
-        self.client.after_handshake(set_bwlimitmenu)
+        self.after_handshake(set_bwlimitmenu)
         self.client.on_server_setting_changed("bandwidth-limit", set_bwlimitmenu)
         return bandwidth_limit_menu_item
     def bwitem(self, menu, bwlimit=0):
@@ -608,7 +621,7 @@ class GTKTrayMenuBase(MenuHelper):
             else:
                 self.encodings_submenu = self.make_encodingssubmenu()
                 encodings.set_submenu(self.encodings_submenu)
-        self.client.after_handshake(set_encodingsmenuitem)
+        self.after_handshake(set_encodingsmenuitem)
         #FUGLY warning: we want to update the menu if we get an 'encodings' packet,
         #so we inject our handler:
         saved_process_encodings = getattr(self.client, "_process_encodings")
@@ -621,14 +634,30 @@ class GTKTrayMenuBase(MenuHelper):
             self.client._process_encodings = process_encodings
         return encodings
 
-    def make_encodingssubmenu(self):
+    def get_encoding_options(self):
         server_encodings = list(self.client.server_encodings)
-        encodings = [x for x in PREFERRED_ENCODING_ORDER if x in self.client.get_encodings()]
-        encodings.insert(0, "auto")
+        client_encodings = [x for x in PREFERRED_ENCODING_ORDER if x in self.client.get_encodings()]
+        #separator:
+        client_encodings.insert(0, "-")
+        server_encodings.insert(0, "-")
+        client_encodings.insert(1, "label:Don't use these directly:")
+        server_encodings.insert(1, "label:Don't use these directly:")
+        if "grayscale" in client_encodings and "grayscale" in server_encodings:
+            #move grayscale to the top:
+            client_encodings.remove("grayscale")
+            server_encodings.remove("grayscale")
+            client_encodings.insert(0, "grayscale")
+            server_encodings.insert(0, "grayscale")
+        #auto at the very top:
+        client_encodings.insert(0, "auto")
         server_encodings.insert(0, "auto")
+        return client_encodings, server_encodings
+
+    def make_encodingssubmenu(self):
+        client_encodings, server_encodings = self.get_encoding_options()
         encodings_submenu = make_encodingsmenu(self.get_current_encoding,
                                                self.set_current_encoding,
-                                               encodings, server_encodings)
+                                               client_encodings, server_encodings)
         return encodings_submenu
 
     def get_current_encoding(self):
@@ -649,7 +678,7 @@ class GTKTrayMenuBase(MenuHelper):
     def make_scalingmenu(self):
         scaling_submenu = Gtk.Menu()
         scaling_submenu.updating = False
-        from xpra.client.mixins.display import SCALING_OPTIONS
+        from xpra.client.scaling_parser import SCALING_OPTIONS
         for x in SCALING_OPTIONS:
             scaling_submenu.append(self.make_scalingvaluemenuitem(scaling_submenu, x))
         def scaling_changed(*args):
@@ -687,7 +716,7 @@ class GTKTrayMenuBase(MenuHelper):
             scaling_submenu.updating = True
             c.set_active(scalecmp(scalingvalue))
             scaling_submenu.updating = False
-        self.client.after_handshake(set_active_state)
+        self.after_handshake(set_active_state)
         return c
 
 
@@ -697,7 +726,7 @@ class GTKTrayMenuBase(MenuHelper):
         def may_enable_qualitymenu(*_args):
             self.quality.set_submenu(self.make_qualitysubmenu())
             self.set_qualitymenu()
-        self.client.after_handshake(may_enable_qualitymenu)
+        self.after_handshake(may_enable_qualitymenu)
         return self.quality
 
     def make_qualitysubmenu(self):
@@ -747,7 +776,7 @@ class GTKTrayMenuBase(MenuHelper):
         def may_enable_speedmenu(*_args):
             self.speed.set_submenu(self.make_speedsubmenu())
             self.set_speedmenu()
-        self.client.after_handshake(may_enable_speedmenu)
+        self.after_handshake(may_enable_speedmenu)
         return self.speed
 
     def make_speedsubmenu(self):
@@ -817,7 +846,7 @@ class GTKTrayMenuBase(MenuHelper):
                 return
             set_sensitive(speaker, True)
             speaker.set_submenu(self.make_soundsubmenu(is_speaker_on, self.spk_on, self.spk_off, "speaker-changed"))
-        self.client.after_handshake(speaker_state)
+        self.after_handshake(speaker_state)
         return speaker
 
     def mic_on(self, *args):
@@ -843,7 +872,7 @@ class GTKTrayMenuBase(MenuHelper):
             set_sensitive(microphone, True)
             microphone.set_submenu(self.make_soundsubmenu(is_microphone_on,
                                                           self.mic_on, self.mic_off, "microphone-changed"))
-        self.client.after_handshake(microphone_state)
+        self.after_handshake(microphone_state)
         return microphone
 
     def sound_submenu_activate(self, item, menu, cb):
@@ -884,7 +913,7 @@ class GTKTrayMenuBase(MenuHelper):
                     ensure_item_selected(menu, off)
             menu.ignore_events = False
         self.client.connect(client_signal, update_soundsubmenu_state)
-        self.client.after_handshake(update_soundsubmenu_state)
+        self.after_handshake(update_soundsubmenu_state)
         menu.show_all()
         return menu
 
@@ -931,7 +960,7 @@ class GTKTrayMenuBase(MenuHelper):
                 sync.set_tooltip_text("video-sync requires speaker forwarding")
                 return
             set_sensitive(sync, True)
-        self.client.after_handshake(set_avsyncmenu)
+        self.after_handshake(set_avsyncmenu)
         return sync
 
 
@@ -1038,7 +1067,7 @@ class GTKTrayMenuBase(MenuHelper):
             menu.ignore_events = False
         self.client.connect("webcam-changed", webcam_changed)
         set_sensitive(webcam, False)
-        self.client.after_handshake(webcam_changed)
+        self.after_handshake(webcam_changed)
         self.client.on_server_setting_changed("webcam", webcam_changed)
         return webcam
 
@@ -1079,8 +1108,8 @@ class GTKTrayMenuBase(MenuHelper):
                     kh.update()
                     kh.send_layout()
                     kh.send_keymap()
-                    log.info("%s: %s", msg, kh.layout_str())
-            l = self.checkitem(title, set_layout, active)
+                    log.info(f"{msg}: {kh.layout_str()}")
+            l = self.checkitem(str(title), set_layout, active)
             l.set_draw_as_radio(True)
             l.keyboard_layout = layout
             l.keyboard_variant = variant
@@ -1088,14 +1117,19 @@ class GTKTrayMenuBase(MenuHelper):
         def keysort(key):
             c,l = key
             return c.lower()+l.lower()
-        layout, layouts, variant, variants, _ = self.client.keyboard_helper.get_layout_spec()
+        def variants_submenu(layout, variants):
+            #just show all the variants to choose from this layout
+            default_layout = kbitem(f"{layout} - Default", layout, "", True)
+            self.layout_submenu.append(default_layout)
+            for v in variants:
+                self.layout_submenu.append(kbitem(f"{layout} - {v}", layout, v))
+        kh = self.client.keyboard_helper
+        layout, layouts, variant, variants, _ = kh.get_layout_spec()
         layout = bytestostr(layout)
         layouts = tuple(bytestostr(x) for x in layouts)
         variant = bytestostr(variant or b"")
         variants = tuple(bytestostr(x) for x in variants)
-        log("make_layoutsmenuitem() layout=%s, layouts=%s, variant=%s, variants=%s",
-            layout, layouts, variant, variants)
-        full_layout_list = False
+        log(f"make_layoutsmenuitem() layout={layout}, layouts={layouts}, variant={variant}, variants={variants}")
         if len(layouts)>1:
             log("keyboard layouts: %s", ",".join(bytestostr(x) for x in layouts))
             #log after removing dupes:
@@ -1106,21 +1140,31 @@ class GTKTrayMenuBase(MenuHelper):
             auto = kbitem("Auto", "Auto", "", True)
             self.layout_submenu.append(auto)
             if layout:
-                self.layout_submenu.append(kbitem("%s" % layout, layout, ""))
+                self.layout_submenu.append(kbitem(layout, layout, ""))
             if variants:
                 for v in variants:
-                    self.layout_submenu.append(kbitem("%s - %s" % (layout, v), layout, v))
+                    self.layout_submenu.append(kbitem(f"{layout} - {v}", layout, v))
             for l in uniq(layouts):
                 if l!=layout:
-                    self.layout_submenu.append(kbitem("%s" % l, l, ""))
-        elif layout and variants and len(variants)>1:
-            #just show all the variants to choose from this layout
-            default = kbitem("%s - Default" % layout, layout, "", True)
-            self.layout_submenu.append(default)
-            for v in variants:
-                self.layout_submenu.append(kbitem("%s - %s" % (layout, v), layout, v))
+                    self.layout_submenu.append(kbitem(l, l, ""))
+        elif layout and len(variants)>1:
+            variants_submenu(layout, variants)
+        elif layout or kh.query_struct:
+            l = layout or kh.query_struct.get("layout", "")
+            from xpra.keyboard.layouts import LAYOUT_VARIANTS
+            variants = LAYOUT_VARIANTS.get(l) if l else ()
+            if variants:
+                variants_submenu(l, variants)
+            else:
+                if l:
+                    keyboard.set_tooltip_text(f"Detected {l!r}")
+                set_sensitive(keyboard, False)
+                return keyboard
+        elif not FULL_LAYOUT_LIST:
+            keyboard.set_tooltip_text(f"No keyboard layouts detected")
+            set_sensitive(keyboard, False)
+            return keyboard
         else:
-            full_layout_list = True
             from xpra.keyboard.layouts import X11_LAYOUTS
             #show all options to choose from:
             sorted_keys = list(X11_LAYOUTS.keys())
@@ -1128,36 +1172,66 @@ class GTKTrayMenuBase(MenuHelper):
             for key in sorted_keys:
                 country,language = key
                 layout,variants = X11_LAYOUTS.get(key)
-                name = "%s - %s" % (country, language)
+                name = f"{country} - {language}"
                 if len(variants)>1:
                     #sub-menu for each variant:
                     variant = self.menuitem(name, tooltip=layout)
                     variant_submenu = Gtk.Menu()
                     variant.set_submenu(variant_submenu)
                     self.layout_submenu.append(variant)
-                    variant_submenu.append(kbitem("%s - Default" % layout, layout, None))
+                    variant_submenu.append(kbitem(f"{layout} - Default", layout, None))
                     for v in variants:
-                        variant_submenu.append(kbitem("%s - %s" % (layout, v), layout, v))
+                        variant_submenu.append(kbitem(f"{layout} - {v}", layout, v))
                 else:
                     #no variants:
                     self.layout_submenu.append(kbitem(name, layout, None))
-        keyboard_helper = self.client.keyboard_helper
-        def set_layout_enabled(*args):
-            log("set_layout_enabled%s full_layout_list=%s",
-                     args, full_layout_list)
-            log("set_layout_enabled%s layout=%s, print=%s, query=%s",
-                     args, keyboard_helper.xkbmap_layout, keyboard_helper.xkbmap_print, keyboard_helper.xkbmap_query)
-            if full_layout_list and \
-            (keyboard_helper.xkbmap_layout or keyboard_helper.xkbmap_print or keyboard_helper.xkbmap_query):
-                #we have detected a layout
-                #so no need to show the user the huge layout list
-                keyboard.hide()
-                return
-            set_sensitive(keyboard, True)
-        self.client.after_handshake(set_layout_enabled)
+        self.after_handshake(set_sensitive, keyboard, True)
         return keyboard
 
 
+    def make_monitorsmenuitem(self):
+        monitors_menu_item = self.handshake_menuitem("Monitors", "display.png")
+        menu = Gtk.Menu()
+        monitors_menu_item.set_submenu(menu)
+        def populate_monitors(*args):
+            log("populate_monitors%s client server_multi_monitors=%s, server_monitors=%s",
+                     args, self.client.server_multi_monitors, self.client.server_monitors)
+            if not self.client.server_multi_monitors or not self.client.server_monitors:
+                monitors_menu_item.hide()
+                return
+            for x in menu.get_children():
+                menu.remove(x)
+            def monitor_changed(mitem, index):
+                log("monitor_changed(%s, %s)", mitem, index)
+                self.client.send_remove_monitor(index)
+            for i, monitor in self.client.server_monitors.items():
+                mitem = Gtk.CheckMenuItem(label=monitor.get("name", "VFB-%i" % i))
+                mitem.set_active(True)
+                mitem.set_draw_as_radio(True)
+                mitem.connect("toggled", monitor_changed, i)
+                menu.append(mitem)
+            #and finally, an entry for adding a new monitor:
+            add_monitor_item = self.menuitem("Add a monitor")
+            resolutions_menu = Gtk.Menu()
+            add_monitor_item.set_submenu(resolutions_menu)
+            def add_monitor(mitem, resolution):
+                log("add_monitor(%s, %s)", mitem, resolution)
+                #older servers may not have all the aliases:
+                resolution = RESOLUTION_ALIASES.get(resolution, resolution)
+                self.client.send_add_monitor(resolution)
+            for resolution in NEW_MONITOR_RESOLUTIONS:
+                mitem = self.menuitem(resolution)
+                mitem.connect("activate", add_monitor, resolution)
+                resolutions_menu.append(mitem)
+            resolutions_menu.show_all()
+            menu.append(add_monitor_item)
+            menu.show_all()
+        self.client.on_server_setting_changed("monitors", populate_monitors)
+        self.after_handshake(populate_monitors)
+        monitors_menu_item.show_all()
+        return monitors_menu_item
+
+
     def make_windowsmenuitem(self):
         windows_menu_item = self.handshake_menuitem("Windows", "windows.png")
         menu = Gtk.Menu()
@@ -1279,7 +1353,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.servercommands.set_tooltip_text(SERVER_NOT_SUPPORTED)
             else:
                 self.servercommands.set_tooltip_text("")
-        self.client.after_handshake(enable_servercommands)
+        self.after_handshake(enable_servercommands)
         return self.servercommands
 
     def make_runcommandmenuitem(self):
@@ -1293,7 +1367,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.startnewcommand.set_tooltip_text("Not supported or enabled on the server")
             else:
                 self.startnewcommand.set_tooltip_text("")
-        self.client.after_handshake(enable_start_new_command)
+        self.after_handshake(enable_start_new_command)
         self.client.on_server_setting_changed("start-new-commands", enable_start_new_command)
         return self.startnewcommand
 
@@ -1308,7 +1382,7 @@ class GTKTrayMenuBase(MenuHelper):
                        self.client.remote_open_files_ask or
                        self.client.remote_open_url_ask)
             set_sensitive(self.transfers, has_ask)
-        self.client.after_handshake(enable_transfers)
+        self.after_handshake(enable_transfers)
         return self.transfers
 
     def make_uploadmenuitem(self):
@@ -1320,7 +1394,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.upload.set_tooltip_text(SERVER_NOT_SUPPORTED)
             else:
                 self.upload.set_tooltip_text("Send a file to the server")
-        self.client.after_handshake(enable_upload)
+        self.after_handshake(enable_upload)
         return self.upload
 
     def make_downloadmenuitem(self):
@@ -1335,7 +1409,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.download.set_tooltip_text("'send-file' subcommand is not supported by the server")
             else:
                 self.download.set_tooltip_text("Send a file to the server")
-        self.client.after_handshake(enable_download)
+        self.after_handshake(enable_download)
         return self.download
 
 
@@ -1352,7 +1426,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.download_log.set_tooltip_text("Server does not expose its log-file")
             else:
                 self.download_log.set_tooltip_text("Download the server log")
-        self.client.after_handshake(enable_download)
+        self.after_handshake(enable_download)
         return self.download_log
 
 
@@ -1386,7 +1460,7 @@ class GTKTrayMenuBase(MenuHelper):
                 self.shutdown.set_tooltip_text("Disabled by the server")
             else:
                 self.shutdown.set_tooltip_text("Shutdown this server session")
-        self.client.after_handshake(enable_shutdown)
+        self.after_handshake(enable_shutdown)
         self.client.on_server_setting_changed("client-shutdown", enable_shutdown)
         return self.shutdown
 
@@ -1394,6 +1468,10 @@ class GTKTrayMenuBase(MenuHelper):
         start_menu_item = self.handshake_menuitem("Start", "start.png")
         start_menu_item.show()
         def update_menu_data():
+            if not self.client.start_new_commands:
+                set_sensitive(start_menu_item, False)
+                start_menu_item.set_tooltip_text("Starting new commands is disabled")
+                return
             if not self.client.server_start_new_commands:
                 set_sensitive(start_menu_item, False)
                 start_menu_item.set_tooltip_text("This server does not support starting new commands")
@@ -1412,7 +1490,7 @@ class GTKTrayMenuBase(MenuHelper):
                 log("on_xdg_menu_changed(%s, %s)", setting, repr_ellipsized(str(value)))
                 update_menu_data()
             self.client.on_server_setting_changed("xdg-menu", on_xdg_menu_changed)
-        self.client.after_handshake(start_menu_init)
+        self.after_handshake(start_menu_init)
         return start_menu_item
 
     def build_start_menu(self):
