@@ -9,10 +9,10 @@ import subprocess
 from queue import Queue
 
 from xpra.gtk_common.gobject_compat import register_os_signals
-from xpra.util import repr_ellipsized, envint, envbool
+from xpra.util import repr_ellipsized, envint, envbool, csv
 from xpra.net.bytestreams import TwoFileConnection
 from xpra.net.common import ConnectionClosedException, PACKET_TYPES
-from xpra.net.protocol import Protocol
+from xpra.net.protocol import Protocol, CONNECTION_LOST, GIBBERISH
 from xpra.os_util import setbinarymode, SIGNAMES, bytestostr, hexstr, WIN32
 from xpra.child_reaper import getChildReaper
 from xpra.log import Logger
@@ -47,9 +47,9 @@ FLUSH = envbool("XPRA_SUBPROCESS_FLUSH", False)
 
 
 FAULT_RATE = envint("XPRA_WRAPPER_FAULT_INJECTION_RATE")
-def noop(_p):
-    pass
-INJECT_FAULT = noop
+def nofault(_p):
+    """ by default, don't inject any errors """
+INJECT_FAULT = nofault
 if FAULT_RATE>0:
     _counter = 0
     def DO_INJECT_FAULT(p):
@@ -163,16 +163,16 @@ class subprocess_callee:
         #figure out where we read from and write to:
         if self.input_filename=="-":
             #disable stdin buffering:
-            self._input = os.fdopen(sys.stdin.fileno(), 'rb', 0)
+            self._input = os.fdopen(sys.stdin.fileno(), "rb", 0)
             setbinarymode(self._input.fileno())
         else:
-            self._input = open(self.input_filename, 'rb')
+            self._input = open(self.input_filename, "rb")
         if self.output_filename=="-":
             #disable stdout buffering:
-            self._output = os.fdopen(sys.stdout.fileno(), 'wb', 0)
+            self._output = os.fdopen(sys.stdout.fileno(), "wb", 0)
             setbinarymode(self._output.fileno())
         else:
-            self._output = open(self.output_filename, 'wb')
+            self._output = open(self.output_filename, "wb")
         #stdin and stdout wrapper:
         conn = TwoFileConnection(self._output, self._input,
                                  abort_test=None, target=self.name,
@@ -199,19 +199,19 @@ class subprocess_callee:
 
 
     def cleanup(self):
-        pass
+        """ subclasses may override this method """
 
     def stop(self):
-        self.cleanup()
         p = self.protocol
         log("stop() protocol=%s", p)
+        self.cleanup()
         if p:
             self.protocol = None
             p.close()
         self.do_stop()
 
     def do_stop(self):
-        log("stop() stopping mainloop %s", self.mainloop)
+        log("do_stop() stopping mainloop %s", self.mainloop)
         self.mainloop.quit()
 
     def handle_signal(self, sig):
@@ -251,11 +251,11 @@ class subprocess_callee:
 
     def process_packet(self, proto, packet):
         command = bytestostr(packet[0])
-        if command==Protocol.CONNECTION_LOST:
+        if command==CONNECTION_LOST:
             log("connection-lost: %s, calling stop", packet[1:])
             self.net_stop()
             return
-        if command==Protocol.GIBBERISH:
+        if command==GIBBERISH:
             log.warn("gibberish received:")
             log.warn(" %s", repr_ellipsized(packet[1], limit=80))
             log.warn(" stopping")
@@ -272,7 +272,7 @@ class subprocess_callee:
         #make it easier to hookup signals to methods:
         attr = command.replace("-", "_")
         if self.method_whitelist is not None and attr not in self.method_whitelist:
-            log.warn("invalid command: %s (not in whitelist: %s)", attr, self.method_whitelist)
+            log.warn("invalid command %r, not in whitelist: %s", attr, csv(self.method_whitelist))
             return
         wo = self.wrapped_object
         if not wo:
@@ -341,8 +341,8 @@ class subprocess_caller:
         self.signal_callbacks = {}
         self.large_packets = []
         #hook a default packet handlers:
-        self.connect(Protocol.CONNECTION_LOST, self.connection_lost)
-        self.connect(Protocol.GIBBERISH, self.gibberish)
+        self.connect(CONNECTION_LOST, self.connection_lost)
+        self.connect(GIBBERISH, self.gibberish)
         from gi.repository import GLib
         self.idle_add = GLib.idle_add
         self.timeout_add = GLib.timeout_add
@@ -415,7 +415,8 @@ class subprocess_caller:
                 proc.terminate()
                 self.process = None
             except Exception as e:
-                log.warn("failed to stop the wrapped subprocess %s: %s", proc, e)
+                log("stop_process() proc=%s", proc, exc_info=True)
+                log.warn("Warning: failed to stop the wrapped subprocess %s: %s", proc, e)
 
     def stop_protocol(self):
         p = self.protocol
@@ -425,7 +426,8 @@ class subprocess_caller:
             try:
                 p.close()
             except Exception as e:
-                log.warn("failed to close the subprocess connection: %s", p, e)
+                log("stop_protocol() protocol=%s", p, exc_info=True)
+                log.warn("Warning: failed to close the subprocess connection %s: %s", p, e)
 
 
     def connection_lost(self, *args):
